==> isolate_contactor-2.1.0.tar.gz: lib/src/isolate_contactor_controller/web_platform/isolate_contactor_controller_web_worker.dart <==
import 'dart:async';
import 'dart:html';

import 'package:isolate_contactor/src/utils/exception.dart';
import 'package:isolate_contactor/src/utils/utils.dart';

import '../isolate_contactor_controller_web.dart';

class IsolateContactorControllerImplWorker<T>
    implements IsolateContactorControllerImpl<T> {
  late Worker _delegate;

  final StreamController<T> _mainStreamController =
      StreamController.broadcast();
  final StreamController _isolateStreamController =
      StreamController.broadcast();

  final Function()? onDispose;
  final T Function(dynamic value) workerConverter;
  dynamic _initialParams;

  IsolateContactorControllerImplWorker(
    dynamic params, {
    this.onDispose,
    required T Function(dynamic) converter, // Converter for native
    required this.workerConverter, // Converter for Worker (Web Only)
  }) {
    if (params is List) {
      _delegate = params.last.controller;
      _initialParams = params.first;
    } else {
      _delegate = params;
    }

    _delegate.onMessage.listen((event) {
      if (event.data == IsolateState.dispose) {
        onDispose!();
        close();
        return;
      }

      if (IsolateException.isValidObject(event.data)) {
        final exception = IsolateException.fromJson(event.data);
        _mainStreamController.addError(
            exception.error.toString(), StackTrace.empty);
        return;
      }

      // Decode json from string which sent from isolate
      _mainStreamController.add(workerConverter(event.data));
    });
  }

  @override
  Worker get controller => _delegate;

  /// Get initial params for `createOwnIsolate`
  @override
  dynamic get initialParams => _initialParams;

  @override
  Stream<T> get onMessage => _mainStreamController.stream;

  @override
  Stream get onIsolateMessage => _isolateStreamController.stream;

  @override
  void sendIsolate(dynamic message) {
    try {
      _delegate.postMessage(message);
    } catch (_) {
      // The delegate may be closed
    }
  }

  @override
  void sendResult(T message) => throw UnimplementedError();

  @override
  Future<void> close() async {
    _delegate.terminate();
    await Future.wait([
      _mainStreamController.close(),
      _isolateStreamController.close(),
    ]);
  }
}

==> isolate_contactor-4.0.0-rc.1.tar.gz: lib/src/isolate_contactor_controller/web_platform/isolate_contactor_controller_web_worker.dart <==
import 'dart:async';
import 'dart:html';

import 'package:isolate_contactor/src/utils/exception.dart';
import 'package:isolate_contactor/src/utils/utils.dart';

import '../isolate_contactor_controller_web.dart';

class IsolateContactorControllerImplWorker<R, P>
    implements IsolateContactorControllerImpl<R, P> {
  late Worker _delegate;

  final StreamController<R> _mainStreamController =
      StreamController.broadcast();
  final StreamController<P> _isolateStreamController =
      StreamController.broadcast();

  final void Function()? onDispose;
  final R Function(dynamic value) workerConverter;
  dynamic _initialParams;

  IsolateContactorControllerImplWorker(
    dynamic params, {
    this.onDispose,
    required R Function(dynamic) converter, // Converter for native
    required this.workerConverter, // Converter for Worker (Web Only)
  }) {
    if (params is List) {
      _delegate = params.last.controller;
      _initialParams = params.first;
    } else {
      _delegate = params;
    }

    _delegate.onMessage.listen((event) {
      if (event.data == IsolateState.dispose) {
        onDispose!();
        close();
        return;
      }

      if (IsolateException.isValidObject(event.data)) {
        final exception = IsolateException.fromJson(event.data);
        _mainStreamController.addError(
            exception.error.toString(), StackTrace.empty);
        return;
      }

      // Decode json from string which sent from isolate
      _mainStreamController.add(workerConverter(event.data));
    });
  }

  @override
  Worker get controller => _delegate;

  /// Get initial params for `createOwnIsolate`
  @override
  dynamic get initialParams => _initialParams;

  @override
  Stream<R> get onMessage => _mainStreamController.stream;

  @override
  Stream<P> get onIsolateMessage => _isolateStreamController.stream;

  @override
  void sendIsolate(P message) {
    try {
      _delegate.postMessage(message);
    } catch (_) {
      // The delegate may be closed
    }
  }

  @override
  void sendIsolateState(Object state) {
    try {
      _delegate.postMessage(state);
    } catch (_) {
      // The delegate may be closed
    }
  }

  @override
  void sendResult(R message) => throw UnimplementedError();

  @override
  Future<void> close() async {
    _delegate.terminate();
    await Future.wait([
      _mainStreamController.close(),
      _isolateStreamController.close(),
    ]);
  }
}
